<?php

namespace Drupal\commerce_opayo_pi\Plugin\Commerce\CheckoutFlow;

use Drupal\commerce_checkout\Plugin\Commerce\CheckoutFlow\CheckoutFlowWithPanesBase;
use Drupal\commerce_checkout\Event\CheckoutEvents;
use Drupal\commerce_order\Event\OrderEvent;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Exception;

/**
 * @CommerceCheckoutFlow(
 *  id = "opayo_checkout_flow",
 *  label = @Translation("Opayo checkout flow"),
 * )
 */
class OpayoCheckoutFlow extends CheckoutFlowWithPanesBase {


  /**
   * {@inheritdoc}
   */
  public function getSteps() {
    // Note that previous_label and next_label are not the labels
    // shown on the step itself. Instead, they are the labels shown
    // when going back to the step, or proceeding to the step.
    return [
      'login' => [
        'label' => $this->t('Login'),
        'previous_label' => $this->t('Go back'),
        'has_sidebar' => FALSE,
      ],
      'order_information' => [
        'label' => $this->t('Order information'),
        'has_sidebar' => TRUE,
        'previous_label' => $this->t('Go back'),
      ],
      'review' => [
        'label' => $this->t('Review'),
        'next_label' => $this->t('Continue to review'),
        'previous_label' => $this->t('Go back'),
        'has_sidebar' => TRUE,
      ],
      'payment' => [
        'label' => $this->t('Payment'),
        'next_label' => $this->t('Pay and complete purchase'),
        'has_sidebar' => FALSE,
        'hidden' => TRUE,
      ],
      'threedsecure' => [
        'label' => $this->t('3D Secure'),
        'next_label' => $this->t('Continue'),
        'previous_label' => $this->t('Go back'),
        'has_sidebar' => TRUE,
      ],
      'complete' => [
        'label' => $this->t('Complete'),
        'next_label' => $this->t('Complete checkout'),
        'has_sidebar' => FALSE,
      ],
    ];
  }


  /**
   * {@inheritdoc}
   */
  protected function onStepChange($step_id) {
    $order = $this->getOrder();

    // Lock the order while on the 'payment' checkout step. Unlock elsewhere.
    if ($step_id == 'payment' || $step_id == 'threedsecure') {
      $order->lock();
    }
    elseif ($step_id != 'payment' && $step_id != 'threedsecure') {
      $order->unlock();
    }
    // Place the order.
    if ($step_id == 'complete' && $order->getState()->getId() == 'draft') {
      // Notify other modules.
      $event = new OrderEvent($order);
      $this->eventDispatcher->dispatch($event, CheckoutEvents::COMPLETION);
      $order->getState()->applyTransitionById('place');
    }
  }


  public function onStepChangeCustom($step_id) {
    $this->onStepChange($step_id);
  }

  public function setOrder($order) {
    $this->order = $order;
  }


  /**
   * {@inheritdoc}
   */
  public function getNextStepId($step_id) {
    if ($step_id == 'complete') {
      // There is no step after complete, return 'complete'
      return $step_id;
    }
    else {
      return parent::getNextStepId($step_id);
    }
    return NULL;
  }


}
