<?php

namespace Drupal\commerce_opayo_pi\Plugin\Commerce\PaymentMethodType;

use Drupal\entity\BundleFieldDefinition;
use Drupal\commerce_payment\CreditCard as CreditCardHelper;
use Drupal\commerce_payment\Entity\PaymentMethodInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentMethodType\PaymentMethodTypeBase;

/**
 * Custom credit card payment type for Opayo Pi
 *
 * @CommercePaymentMethodType(
 *   id = "credit_card_opayo",
 *   label = @Translation("new Credit/Debit card"),
 * )
 */
class CreditDebitCard extends \Drupal\commerce_payment\Plugin\Commerce\PaymentMethodType\CreditCard {

  /**
   * {@inheritdoc}
   */
  public function buildLabel(PaymentMethodInterface $payment_method) {

    $remoteId = $payment_method->getRemoteId();
    if (!isset($remoteId))
      return $this->t('new Credit/Debit card');
    else {
      // Previously entered payment method
      $payment_expired_suffix = $payment_method->isExpired() ? ' (verification expired)' : '';

      if (isset($payment_method->card_type->value) && isset($payment_method->card_number->value)) {
        $card_type = CreditCardHelper::getType($payment_method->card_type->value);
        $args = [
          '@card_type' => $card_type->getLabel(),
          '@card_number' => $payment_method->card_number->value,
        ];
        return $this->t('@card_type ending in @card_number' . $payment_expired_suffix, $args);
      }
      else {
        return $this->t('Your card' . $payment_expired_suffix);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildFieldDefinitions() {
    $fields = parent::buildFieldDefinitions();
    return $fields;
  }

}
